#!/bin/bash

NSX_MANAGER_ADDRESS="${NSX_MANAGER_ADDRESS:?Error: NSX_MANAGER_ADDRESS environment variable is not set}"
MAX_RETRIES=10
RETRY_INTERVAL=5

API_ENDPOINT="https://$NSX_MANAGER_ADDRESS/policy/api/v1/config-streaming-agent/consumers/NAPP"
TOI_REQUEST_BODY='{ "consumer_name":"NAPP", "version":"NAPP_INTELLIGENCE_421", "streaming_type":"KAFKA", "groups":[ { "streaming_address":"nsx2pace-config", "tables":[ { "name":"Domain", "versions":[ "CLASSIC" ] }, { "name":"Group", "versions":[ "CLASSIC" ] }, { "name":"CommunicationMap", "versions":[ "CLASSIC" ] }, { "name":"CommunicationEntry", "versions":[ "CLASSIC" ] }, { "name":"Service", "versions":[ "CLASSIC" ] }, { "name":"ServiceEntry", "versions":[ "CLASSIC" ] }, { "name":"ContextProfile", "versions":[ "CLASSIC" ] }, { "name":"SegmentPort", "versions":[ "CLASSIC" ] }, { "name":"HostTransportNode", "versions":[ "CLASSIC" ] }, { "name":"HostModelMsg", "versions":[ "CLASSIC" ] }, { "name":"VirtualMachineContainerMsg", "versions":[ "CLASSIC" ] }, { "name":"VmPropertiesMsg", "versions":[ "CLASSIC" ] }, { "name":"VniMsg", "versions":[ "CLASSIC" ] }, { "name":"ContainerClusterMsg", "versions":[ "CLASSIC" ] }, { "name":"ContainerProjectMsg", "versions":[ "CLASSIC" ] }, { "name":"ContainerApplicationMsg", "versions":[ "CLASSIC" ] }, { "name":"ContainerApplicationInstanceMsg", "versions":[ "CLASSIC" ] }, { "name":"IpDiscoveryComputedBinding", "versions":[ "CLASSIC" ] }, { "name":"GroupingNotification", "versions":[ "CLASSIC" ] }, { "name":"IdsProfile", "versions":[ "CLASSIC" ] }, { "name":"GlobalIdsSignature", "versions":[ "CLASSIC" ] }, { "name":"FirewallRule", "versions":[ "CLASSIC" ] }, { "name":"CommonAgentHostConfigProfile", "versions":[ "CLASSIC" ] } ] }, { "streaming_address":"nsx2metric-config", "tables":[ { "name":"PolicyEdgeNode", "versions":[ "CLASSIC" ] }, { "name":"ProviderInterface", "versions":[ "CLASSIC" ] }, { "name":"HostTransportNode", "versions":[ "CLASSIC" ] }, { "name":"NetworkInterface", "versions":[ "CLASSIC" ] }, { "name":"ClusterConfiguration", "versions":[ "CLASSIC" ] }, { "name":"IpSecVpnSession", "versions":[ "CLASSIC" ] }, { "name":"Provider", "versions":[ "CLASSIC" ] }, { "name":"SegmentPort", "versions":[ "CLASSIC" ] }, { "name":"CommunicationEntry", "versions":[ "CLASSIC" ] }, { "name":"IdsRule", "versions":[ "CLASSIC" ] }, { "name":"TlsGatewayRule", "versions":[ "CLASSIC" ] }, { "name":"Network", "versions":[ "CLASSIC" ] }, { "name":"ServiceInterface", "versions":[ "CLASSIC" ] }, { "name":"IdsGatewayRule", "versions":[ "CLASSIC" ] } ] } ] }'

make_api_call() {
  curl -o /dev/null --write-out "%{http_code}" -k --location --request PUT "$API_ENDPOINT" \
    --header 'Content-Type: application/json' \
    --data-raw "$TOI_REQUEST_BODY" \
    --cacert "$CA_CERT_PATH" \
    --cert "$EGRESS_CERT_PATH" \
    --key "$EGRESS_KEY_PATH"
}

# Retry loop
for i in $(seq 1 $MAX_RETRIES); do
  echo "Attempt $i/$MAX_RETRIES: Calling API endpoint $API_ENDPOINT"
  http_status=$(make_api_call)
  echo "Response: $http_status"

  if [[ "$http_status" == "200" || "$http_status" == "201" ]]; then
    echo "API call successful."
    exit 0
  elif [[ "$http_status" == "404" ]]; then
    echo "API returned 404 Not Found. This API requires NSX Manager 4.2.1 or higher. Exiting."
    exit 0
  else
    echo "API call failed with status code $http_status. Retrying in $RETRY_INTERVAL seconds..."
    sleep $RETRY_INTERVAL
  fi
done

echo "API call failed after $MAX_RETRIES attempts. Exiting."
exit 1
