import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
import json
import time
import traceback
import logging
import os


logging.basicConfig(level=logging.DEBUG)


# overlord and coordinator are merged into coordinator pod/s, this env variable is not getting set by us so
# need to use druid coordinator values here to make sure url is correct
druid_overlord_host = os.environ.get('DRUID_COORDINATOR_SERVICE_HOST')
druid_overlord_port = os.environ.get('DRUID_COORDINATOR_SERVICE_PORT')
druid_url = "https://%s:%s/druid/indexer/v1/supervisor/<supervisorId>/status" % (druid_overlord_host, druid_overlord_port)

# prepare session, max_retries applies only to failed DNS lookups, socket connections and connection timeouts,
# never to requests where data has made it to the server.
retry_strategy = Retry(total=3)
adapter = HTTPAdapter(max_retries=retry_strategy)
httpSession = requests.Session()
httpSession.verify = False
httpSession.mount("http", adapter)

headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}
httpSession.headers = headers


def parseDruidSupervisorStatus(resp, supervisor_id):
    logging.info(resp.text)
    respJson = json.loads(resp.text)
    status = respJson["payload"]["state"]
    logging.info("Supervisor: " + supervisor_id + " status: " + status)
    if status == "RUNNING":
        return True
    else:
        logging.info("Supervisor " + supervisor_id + " is not ready")
        return False


def checkingSupervisorStatus(supervisor_id):
    is_supervisor_up = False
    sleep_time = 30
    time_elapsed = 0

    for retry in range(0, 10):
        if retry > 0:
            time.sleep(sleep_time)
            time_elapsed += 30
        try:
            resp = httpSession.get(url=druid_url.replace("<supervisorId>", supervisor_id), timeout=60)
            if parseDruidSupervisorStatus(resp, supervisor_id):
                logging.info("Druid supervisor " + supervisor_id + " for nsx config is running after " +
                             str(time_elapsed) + " seconds ")
                is_supervisor_up = True
                break
        except:
            logging.error("Supervisor " + supervisor_id + "is not ready after " + str(time_elapsed) + " seconds ")
            traceback.print_exc()
            continue

    if not is_supervisor_up:
        logging.error("Supervisor " + supervisor_id + " is not ready after timeout 300 seconds")
        exit(1)


supervisors = ["pace2druid_policy_intent_config", "pace2druid_manager_realization_config"]

for supervisor in supervisors:
    logging.info('==============Checking the ' + supervisor + ' status=============')
    checkingSupervisorStatus(supervisor)
    logging.info('==============' + supervisor + ' supervisor is in RUNNING state=============')

logging.info("All supervisors are up")
exit(0)
