--
-- Schema upgrade to longview
--
-- Columns added to detector table:
--   - site_id (defaults to a value in dgs_sites table)
--   - org_id (defaults to a value in value.yaml)
--   - new composite key of (name, site_id, org_id)
--   - last_successful_run_time (defaults to null)
--
-- Columns added to event table:
--   - site_ids (defaults to a singleton list with a value from dgs_sites table)
--   - org_id (defaults to a value in value.yaml)
--
-- Upgrade is idempotent
-- Upgrade assumes there is only one site in the dgs_sites table

-- upgrade detector table
ALTER TABLE detector ADD COLUMN IF NOT EXISTS org_id varchar(255);
UPDATE detector SET org_id = '{{ .Values.global.tenantOrgId }}';
ALTER TABLE detector ALTER COLUMN org_id SET NOT NULL;

ALTER TABLE detector ADD COLUMN IF NOT EXISTS site_id varchar(255);
UPDATE detector SET site_id = dgs_sites.site_id FROM dgs_sites;
ALTER TABLE detector ALTER COLUMN site_id SET NOT NULL;

ALTER TABLE detector DROP CONSTRAINT detector_pkey;
ALTER TABLE detector ADD CONSTRAINT detector_pkey PRIMARY KEY (name, org_id, site_id);
ALTER TABLE detector ADD COLUMN IF NOT EXISTS run_status jsonb NOT NULL DEFAULT '{"message": "", "statusLabel": "NOT_STARTED", "checkpointTime": null, "lastSuccessfulRuntime": 0, "maxBaselineDurationDays": null, "remainingBaselineDurationDays": null}'::jsonb;

-- Update the 'exclusion_settings' column in the 'detector' table to ensure that all exclusions have a 'site' key present after upgrade from 4.1.2 to 4.2.0
UPDATE detector
-- Set the 'exclusion_settings' column to a new value
SET exclusion_settings = jsonb_set(
    -- The existing 'exclusion_settings' value
    exclusion_settings,
    -- The key to update: 'exclusions'
    '{exclusions}',
    -- A subquery that updates the 'exclusions' array
    -- Loop through all elements in the 'exclusion' and generate a new array (with site
    -- arrays populated with the desired value)
    (
        -- Aggregate (back) the updated 'exclusions' array
        SELECT jsonb_agg(
            CASE
                -- Check if the current element has a 'site' key
                WHEN elem ? 'site' THEN
                    CASE
                        -- If the 'site' key is present but its value is empty, set it to the detector's site_id
                        WHEN elem->>'site' = '' THEN jsonb_set(elem, '{site}', to_jsonb(detector.site_id))
                        -- If the 'site' key is present and its value is not empty, keep the element as is
                        ELSE elem
                    END
                -- If the 'site' key is not present, add it and set it to the detector's site_id
                ELSE
                    jsonb_set(elem, '{site}', to_jsonb(detector.site_id))
            END
        )
        -- Iterate over each element in the 'exclusions' array
        FROM jsonb_array_elements(exclusion_settings->'exclusions') AS elem
    )
);

-- upgrade event table
ALTER TABLE event ADD COLUMN IF NOT EXISTS org_id varchar(255);
UPDATE event SET org_id = '{{ .Values.global.tenantOrgId }}';

ALTER TABLE event ADD COLUMN IF NOT EXISTS site_ids text[];
UPDATE event SET site_ids = string_to_array(dgs_sites.site_id, ',') FROM dgs_sites;
