{{/*
Expand the name of the chart.
*/}}
{{- define "nta-server.name" -}}
{{- default .Chart.Name .Values.nameOverride | trunc 63 | trimSuffix "-" }}
{{- end }}

{{/*
Create a default fully qualified app name.
We truncate at 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec).
If release name contains chart name it will be used as a full name.
*/}}
{{- define "nta-server.fullname" -}}
{{- if .Values.fullnameOverride }}
{{- .Values.fullnameOverride | trunc 63 | trimSuffix "-" }}
{{- else }}
{{- $name := default .Chart.Name .Values.nameOverride }}
{{- if contains $name .Release.Name }}
{{- .Release.Name | trunc 63 | trimSuffix "-" }}
{{- else }}
{{- printf "%s-%s" .Release.Name $name | trunc 63 | trimSuffix "-" }}
{{- end }}
{{- end }}
{{- end }}

{/*
Create a default detector name prefix
*/}}
{{- define "nta-detector.prefix" -}}
nta-detector
{{- end }}

{{/*
Create a default fully qualified name for detectors

NEW_DETECTOR: For additional detectors, add their name definitions here
*/}}
{{- define "verticalportscan.name" -}}
{{ template "nta-detector.prefix" . }}-verticalportscan
{{- end }}

{{- define "trafficdrop.name" -}}
{{ template "nta-detector.prefix" . }}-trafficdrop
{{- end }}

{{- define "uncommonport.name" -}}
{{ template "nta-detector.prefix" . }}-uncommonport
{{- end }}

{{- define "llmnrnbtns.name" -}}
{{ template "nta-detector.prefix" . }}-llmnrnbtns
{{- end }}

{{/*
Create chart name and version as used by the chart label.
*/}}
{{- define "nta-server.chart" -}}
{{- printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" }}
{{- end }}

{{/*
Common labels
*/}}
{{- define "nta-server.labels" -}}
helm.sh/chart: {{ include "nta-server.chart" . }}
{{ include "nta-server.selectorLabels" . }}
{{- if .Chart.AppVersion }}
app.kubernetes.io/version: {{ .Chart.AppVersion | quote }}
{{- end }}
app.kubernetes.io/managed-by: {{ .Release.Service }}
vmware/version: {{ .Values.global.vmwareVersion | quote }}
{{- end }}

{{/*
Selector labels
*/}}
{{- define "nta-server.selectorLabels" -}}
app.kubernetes.io/name: {{ include "nta-server.name" . }}
app.kubernetes.io/instance: {{ .Release.Name }}
{{- end }}

{{/*
Selector labels for detectors
*/}}
{{- define "nta-detector.selectorLabels" -}}
app.kubernetes.io/instance: {{ .Release.Name }}
{{- end }}

{{/*
Create the name of the service account to use
*/}}
{{- define "nta-server.serviceAccountName" -}}
{{- if .Values.serviceAccount.create }}
{{- default (include "nta-server.fullname" .) .Values.serviceAccount.name }}
{{- else }}
{{- default "default" .Values.serviceAccount.name }}
{{- end }}
{{- end }}

{{/*
Get default value from dict
*/}}
{{- define "getOrDefault" }}
  {{- $v := .Values }}
  {{- $found := true }}
  {{- range $value := regexSplit "\\." .Path -1 }}
    {{- if hasKey $v $value }}
      {{- $v = get $v $value }}
    {{- else }}
      {{- $found = false }}
    {{- end }}
  {{- end }}
  {{- if eq $found true }}
    {{- $v }}
  {{- else }}
    {{- .Default }}
  {{- end }}
{{- end }}
