{{/* vim: set filetype=mustache: */}}
{{/*
Expand the name of the chart.
*/}}
{{- define "analytics-common-chart.name" -}}
{{- default .Chart.Name .Values.nameOverride | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
Create a default fully qualified app name.
We truncate at 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec).
If release name contains chart name it will be used as a full name.
*/}}
{{- define "analytics-common-chart.fullname" -}}
{{- if .Values.fullnameOverride -}}
{{- .Values.fullnameOverride | trunc 63 | trimSuffix "-" -}}
{{- else -}}
{{- $name := default .Chart.Name .Values.nameOverride -}}
{{- if contains $name .Release.Name -}}
{{- .Release.Name | trunc 63 | trimSuffix "-" -}}
{{- else -}}
{{- printf "%s-%s" .Release.Name $name | trunc 63 | trimSuffix "-" -}}
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
Create chart name and version as used by the chart label.
*/}}
{{- define "analytics-common-chart.chart" -}}
{{- printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
Common labels
*/}}
{{- define "analytics-common-chart.labels" -}}
helm.sh/chart: {{ include "analytics-common-chart.chart" . }}
{{ include "analytics-common-chart.selectorLabels" . }}
{{- if .Chart.AppVersion }}
app.kubernetes.io/version: {{ .Chart.AppVersion | quote }}
{{- end }}
app.kubernetes.io/managed-by: {{ .Release.Service }}
{{- end -}}

{{/*
Selector labels
*/}}
{{- define "analytics-common-chart.selectorLabels" -}}
app.kubernetes.io/name: {{ include "analytics-common-chart.name" . }}
app.kubernetes.io/instance: {{ .Release.Name }}
{{- end -}}

{{/*
Create the name of the service account to use
*/}}
{{- define "analytics-common-chart.serviceAccountName" -}}
{{- if .Values.serviceAccount.create -}}
    {{ default (include "analytics-common-chart.fullname" .) .Values.serviceAccount.name }}
{{- else -}}
    {{ default "default" .Values.serviceAccount.name }}
{{- end -}}
{{- end -}}

{{/*
Get default value from dict
*/}}
{{- define "getOrDefault" }}
  {{- $v := .Values }}
  {{- $found := true }}
  {{- range $value := regexSplit "\\." .Path -1 }}
    {{- if hasKey $v $value }}
      {{- $v = get $v $value }}
    {{- else }}
      {{- $found = false }}
    {{- end }}
  {{- end }}
  {{- if eq $found true }}
    {{- $v }}
  {{- else }}
    {{- .Default }}
  {{- end }}
{{- end }}


{{/* kafka keystore is based on env.
on-prem -> same as keystore generated by cert-manager
else, on VDP -> keystore generated by strimzi operator
*/}}
{{- define "kafka.keystore" -}}
{{- if .Values.global.env.vdp.enabled }}
{{- "/etc/nsxi/store/kafka/keystore.p12" }}
{{- else }}
{{- "/etc/nsxi/store/keystore.p12" }}
{{- end}}
{{- end}}


{{/* kafka truststore is based on env.
on-prem -> same as truststore generated by cert-manager
else, on VDP -> truststore generated by strimzi operator
*/}}
{{- define "kafka.truststore" -}}
{{- if .Values.global.env.vdp.enabled }}
{{- "/etc/nsxi/store/kafka/truststore.p12" }}
{{- else }}
{{- "/etc/nsxi/store/truststore.p12" }}
{{- end}}
{{- end}}


{{/* helper to generated a quotes list of string from a yaml snippet */}}
{{- define "helper.commaString" -}}
{{- $c := 0 | int }}
{{- range $value := .valueList}}
{{- if ne $c 0 -}},{{- end -}}{{- $value.name -}}
{{- $c = add1 $c }}
{{- end}}
{{- end}}


{{/* helper to generated a quotes list of string from a yaml snippet */}}
{{- define "helper.commaStringFromMap" -}}
{{- $c := 0 | int }}
{{- range $key, $value := .valueMap}}
{{- if ne $c 0 -}},{{- end -}}{{- $value -}}
{{- $c = add1 $c }}
{{- end}}
{{- end}}
